// File: SampleDlg.cpp
//
// Description: A sample dialog demonstrating virtual printer integration
// methods and print job information retrieval

#include "StdAfx.h"
#include "Resource.h"
#include "SampleDlg.h"

//---------------------------------------------------------------------------

#define PRINTER_NAME	"Two Pilots Demo Printer"
#define REGISTRY_KEY	"Two Pilots Demo Printer"
 
// Print job info transfer modes
enum TransferModes {
    ThroughCommandLine	= 0,
    ThroughCopydata		= 1,
    ThroughClipboard	= 2
};

//---------------------------------------------------------------------------
// CSampleDlg inplementation
//---------------------------------------------------------------------------

CSampleDlg::CSampleDlg(CWnd* pParent /*=NULL*/)	: CDialog(CSampleDlg::IDD, pParent)
{
	m_iTransferMode = ThroughCommandLine;
	m_iMessageID = -1;
}

//---------------------------------------------------------------------------

void CSampleDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_LISTBOX, m_ListBox);
}

//---------------------------------------------------------------------------

BOOL CSampleDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Load icon
	HICON hIcon = AfxGetApp()->LoadIcon(IDI_MAIN);
	SetIcon(hIcon, TRUE);
	SetIcon(hIcon, FALSE);

	// Set unique window title
	CString sWindowTitle = "Sample Client Application for ";
	sWindowTitle += PRINTER_NAME;
	SetWindowText(sWindowTitle);

	// Get transfer mode related settings from the printer configuration

	HKEY hKey = NULL;
	DWORD dwSize = 0;
	CString sKeyName;

	sKeyName.Format("Software\\%s", REGISTRY_KEY);
	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, sKeyName, 0, KEY_READ, &hKey) == ERROR_SUCCESS)
	{
		RegQueryValueEx(hKey, "Transfer Mode", NULL, NULL,
			(BYTE*) &m_iTransferMode, &(dwSize = sizeof(int)));
		RegQueryValueEx(hKey, "Message ID", NULL, NULL,
			(BYTE*) &m_iMessageID, &(dwSize = sizeof(int)));
		RegQueryValueEx(hKey, "Clipboard Format", NULL, NULL,
			(BYTE*) m_sClipboardFormat.GetBuffer(256), &(dwSize = 256));
		m_sClipboardFormat.ReleaseBuffer();
		RegCloseKey(hKey);
	}

	// If transfer mode is Command Line then get print
	// job info file path from the command line
	if (m_iTransferMode == ThroughCommandLine)
	{
    int pNumArgs=0;
    PWSTR*ppArgv=CommandLineToArgvW(GetCommandLineW(),&pNumArgs);
    if(pNumArgs==2)
        m_sInputIniFilePath = ppArgv[1];			
		// Parse .ini file
    
		CFileFind Find;
		if (Find.FindFile(m_sInputIniFilePath))
			FillInfo();
		else{
			m_ListBox.AddString("Invalid command line parameter!");
      m_ListBox.AddString(m_sInputIniFilePath);
    }
	}
	else
		m_ListBox.AddString("Waiting for a print job...");

	return TRUE;
}

//---------------------------------------------------------------------------

void CSampleDlg::FillInfo()
{
	char szBuf[512] = {0};
	CString sTemp;
	int x;

	// Clear the ListBox
	m_ListBox.ResetContent();

	// Show the transfer mode
	sTemp = "Transfer mode = ";
	switch (m_iTransferMode)
	{
		case ThroughCommandLine: sTemp += "Through the command line"; break;
		case ThroughCopydata: sTemp += "Through the WM_COPYDATA message"; break;
		case ThroughClipboard: sTemp += "Through the clipboard"; break;
		default: sTemp += "Unknown";
	}
	m_ListBox.AddString(sTemp);
	m_ListBox.AddString("");

	// Show print job info file
	x = m_ListBox.AddString(m_sInputIniFilePath);
	m_ListBox.SetItemData(x, 1); // mark the listbox row as "clickable"
	m_ListBox.AddString("");

	// Show device name
	GetPrivateProfileString("Device", "DeviceName", NULL,
		szBuf, sizeof(szBuf), m_sInputIniFilePath);
	sTemp.Format("Device name = %s", szBuf);
	m_ListBox.AddString(sTemp);

	// Show print job (document) name
	GetPrivateProfileString("Document", "Name", NULL,
		szBuf, sizeof(szBuf), m_sInputIniFilePath);
	sTemp.Format("Document name = %s", szBuf);
	m_ListBox.AddString(sTemp);

	// Show document resolution (dpi)
	GetPrivateProfileString("Document", "HorizontalResolution", NULL,
		szBuf, sizeof(szBuf), m_sInputIniFilePath);
	sTemp.Format("Horizontal Resolution = %sdpi", szBuf);
	m_ListBox.AddString(sTemp);
	GetPrivateProfileString("Document", "VerticalResolution", NULL,
		szBuf, sizeof(szBuf), m_sInputIniFilePath);
	sTemp.Format("Vertical Resolution = %sdpi", szBuf);
	m_ListBox.AddString(sTemp);

	// Show page orientation (1 - portrait, 2 - landscape)
	int iOrientation = GetPrivateProfileInt("Document", "Orientation",
		0, m_sInputIniFilePath);
	if (iOrientation == 1) sTemp = "Orientation = Portrait"; else
		if (iOrientation == 2) sTemp = "Orientation = Landscape"; else
			sTemp = "Orientation = Unknown";
	m_ListBox.AddString(sTemp);

	// Show paper form name and size
	GetPrivateProfileString("Document", "PaperSizeName", NULL,
		szBuf, sizeof(szBuf), m_sInputIniFilePath);
	int iPaperWidth = GetPrivateProfileInt("Document", "PaperWidth",
		0, m_sInputIniFilePath);
	int iPaperLength = GetPrivateProfileInt("Document", "PaperLength",
		0, m_sInputIniFilePath);
	sTemp.Format("Paper size = %s (%d x %d mm)", szBuf, iPaperWidth/10, iPaperLength/10);
	m_ListBox.AddString(sTemp);

	// Show printer bin
	GetPrivateProfileString("Document", "BinName", NULL,
		szBuf, sizeof(szBuf), m_sInputIniFilePath);
	sTemp.Format("Paper bin = %s", szBuf);
	m_ListBox.AddString(sTemp);

	// Show print job (document) page count
	GetPrivateProfileString("Document", "Pages", NULL,
		szBuf, sizeof(szBuf), m_sInputIniFilePath);
	sTemp.Format("Document pages = %s", szBuf);
	m_ListBox.AddString(sTemp);

	// Show print job (document) copies count
	GetPrivateProfileString("Document", "Copies", NULL,
		szBuf, sizeof(szBuf), m_sInputIniFilePath);
	sTemp.Format("Copies = %s", szBuf);
	m_ListBox.AddString(sTemp);

	// Show print job (document) collate option
	int iCollate = GetPrivateProfileInt("Document", "Collate", 0, m_sInputIniFilePath);
	if (iCollate == 0) sTemp = "Collate = No"; else
		sTemp = "Collate = Yes";
	m_ListBox.AddString(sTemp);

	// Show generated files
	const char* pszFileTypes[] = {
		"EMF", "BMP", "PNG", "JPEG", "TIFF", "PDF", "SWF", "TXT", NULL};
	int iCount, i, j;

	for (i = 0; pszFileTypes[i]; i++)
	{
		iCount = GetPrivateProfileInt(pszFileTypes[i], "Count", 0, m_sInputIniFilePath);
		if (iCount > 0)
		{
			m_ListBox.AddString("");
			sTemp.Format("%s files (%d):", pszFileTypes[i], iCount);
			m_ListBox.AddString(sTemp);

			for (j = 0; j < iCount; j++)
			{
				sTemp.Format("File%d", j);
				GetPrivateProfileString(pszFileTypes[i], sTemp, NULL,
					szBuf, sizeof(szBuf), m_sInputIniFilePath);
				x = m_ListBox.AddString(szBuf);
				m_ListBox.SetItemData(x, 1); // mark the listbox row as "clickable"
			}
		}
	}
}

//---------------------------------------------------------------------------
// CSampleDlg message map & handlers
//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CSampleDlg, CDialog)
	ON_WM_COPYDATA()
	ON_LBN_DBLCLK(IDC_LISTBOX, OnLbnDblclkListbox)
	ON_BN_CLICKED(IDB_PRINTERPREFS, OnBnClickedPrinterprefs)
	ON_BN_CLICKED(IDB_SERVICE, OnBnClickedService)
END_MESSAGE_MAP()

LRESULT CSampleDlg::WindowProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	// Analyze all messages from WM_APP range, check if the
	// message has been received from the EMF printer
	if ((uMsg >= WM_APP) && (uMsg <= 0xBFFF) && ((int) uMsg == m_iMessageID))
	{
		// Check clipboard format
		if (m_sClipboardFormat.IsEmpty())
			AfxMessageBox("Configuration error: clipboard format is undefined",
				MB_ICONERROR | MB_OK);
		else
		{
			// Register clipboard format
			UINT uClpFmt = RegisterClipboardFormat(m_sClipboardFormat);

			// Open the clipboard
			if (!OpenClipboard())
				AfxMessageBox("Failed to open clipboard", MB_ICONERROR | MB_OK);
			else
			{
				// Read data from the clipboard
				HANDLE h;
				if (!(h = GetClipboardData(uClpFmt)))
				{
					CloseClipboard();
					AfxMessageBox("Failed to get clipboard data", MB_ICONERROR | MB_OK);
				} else {
					m_sInputIniFilePath = (WCHAR*) GlobalLock(h);
					GlobalUnlock(h);
					CloseClipboard();

					// Parse .ini file
					CFileFind Find;
					if (Find.FindFile(m_sInputIniFilePath))
					{
						FillInfo();
						SetForegroundWindow();
					} else {
						CString str;
						str.Format("Ini file %s does not exist", m_sInputIniFilePath);
						AfxMessageBox(str, MB_ICONERROR | MB_OK);
					}
				}
			}
		}

		// Message is handled
		return TRUE;
	}
	// Call parent's window procedure
	return CDialog::WindowProc(uMsg, wParam, lParam);
}

BOOL CSampleDlg::OnCopyData(CWnd* pWnd, COPYDATASTRUCT* pCopyDataStruct)
{
	// If message ID is equal to ID from the COPYDATASTRUCT
	// then the message is received from the EMF printer
	if (m_iMessageID == (int) pCopyDataStruct->dwData)
	{
		// Get the file name from the message data
		m_sInputIniFilePath = (WCHAR*) pCopyDataStruct->lpData;

		// Check .ini file existance and parse its content
		CFileFind Find;
		if (Find.FindFile(m_sInputIniFilePath))
		{
			FillInfo();
		    SetForegroundWindow();
		} else {
			CString str;
			str.Format("Ini file %s does not exist", m_sInputIniFilePath);
			AfxMessageBox(str, MB_ICONERROR | MB_OK);
		}

		// Data is accepted
		return TRUE;
	}
	return FALSE;
}

//---------------------------------------------------------------------------

void CSampleDlg::OnLbnDblclkListbox()
{
	// If the clicked item of the listbox contains path of file the
	// file will be opened with default associated application
	int x = m_ListBox.GetCurSel();
	if ((x != LB_ERR) && (m_ListBox.GetItemData(x) == 1))
	{
		CString sPath;
		m_ListBox.GetText(x, sPath);
		ShellExecute(m_hWnd, "open", sPath, NULL, NULL, SW_SHOWNORMAL);
	}
}

//---------------------------------------------------------------------------

void CSampleDlg::OnBnClickedPrinterprefs()
{
	// Execute properties.exe from the same folder
	if (WinExec("properties.exe", SW_SHOWNORMAL) < 32)
		AfxMessageBox("File properties.exe not found", MB_ICONERROR | MB_OK);
}

//---------------------------------------------------------------------------

void CSampleDlg::OnBnClickedService()
{
	// Open virtual printer home page at the web-site
	ShellExecute(m_hWnd, "open", "http://www.colorpilot.com/emfprinterpilot.html",
		NULL, NULL, SW_SHOWNORMAL);
}
